/* filename:    margpm.g
** description: The program maximizes the posterior density of the 
**              marginal log likelihood
**              with respect to the DSGE model parameters
** created:     11/05/01
*/


/********************************************************
** Load sufficient statistics for dataset, Compute OLS estimates
*/
load path=^datapath yyyy, xxxx, xxyy, nobs;

CC_hat  = inv(XXXX)*XXYY;
SIG_hat = ( YYYY -XXYY'*inv(XXXX)*XXYY )/nobs;

likemax = -0.5*nobs*nvar*ln(2*pi) -0.5*nobs*ln(det(Sig_hat))
                -0.5*sumc(diag(inv(Sig_hat)*(YY-XX*CC_hat)'*(YY-XX*CC_hat)));
                
/********************************************************
** Load information for prior distribution
*/

modelid   = ftos(mspec,"%*.*lf",1,0);
lmodel    = "d" $+ modelid; 

if lamind < 10;
   llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
else;
   llam = ftos(lamind,"%*.*lf",2,0);
endif;

if Tind < 10;
   lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
else;
   lsamp = ftos(Tind,"%*.*lf",2,0);
endif;

priorfile = lmodel $+ "pri.out";
load path = ^priopath prior[npara,3] = ^priorfile;

pmean  = prior[1:npara,1];
pstdd  = prior[1:npara,2];
pshape = prior[1:npara,3];

/* Load transformation scheme for parameters
*/
transfile = "trans.out";
load path=^priopath _trspec[npara,4] = ^transfile;


/******************************************************** 
**      Calculate posterior density at starting values
**
*/
lnpy = -objfcn(invtrans(para));

screen off;
output file= ^outstr on;
"Posterior at Starting Value:     " lnpy;
"Prior Density at Starting Value: " priodens(para,pmean,pstdd,pshape);
"Likelihood fcn at Starting Value:" lnpY-priodens(para,pmean,pstdd,pshape);
"Likelihood at unconstrained max: " likemax;
output file= ^outstr off;
screen on;

/********************************************************
**       Maximize the posterior density
**
*/
x0= invtrans(para) + 0.0*rndn(npara,1);
H0= eye(npara)*1E-4;
nit = 1000;

/* Use personal optimization procedure
*/
{fh,xh,g,H,itct,fcount,retcode} = csminwel(x0,H0,crit,nit,&objfcn);

/********************************************************
** Save parameter estimates
*/
paraest = trans(xh);

oparaest         = postpath $+ "\\" $+ modelid $+ llam $+ lsamp $+"pm";
create fhparaest = ^oparaest with PARAEST, 1, 8;
wr               = writer(fhparaest,paraest);
closeall fhparaest;   


/********************************************************
**       Estimation Output
**
*/
lnpy= marglh( paraest,lambda );
lnprio  = priodens( paraest, pmean, pstdd, pshape);

/* Compute gradient in terms of model parameters
*/
gradmod = g./ diag(gradp(&invtrans,paraest));

screen off;
output file=^outstr on;
"Posterior Mode     " (lnpy+lnprio);
"Likelihood at Mode " lnpy;
"Prior at Mode      " lnprio;
" ";
"Parameter   | Estimate |  Start-V  | Prior Mean | Prior Stdd | Gradient";
outmat = para_names'~paraest~trans(x0)~pmean~pstdd~gradmod;
let mask[1,6] = 0 1 1 1 1 1;
let fmt[6,3] =
   "-*.*s " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4;

d = printfm(outmat,(0~1~1~1~1~1),fmt);
output file=^outstr off;
screen on;


/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (1) = objfcn(para);
local lnpy, lnprio, modelpara;

/* likelihood 
*/
modelpara = trans(para);

lnpy = marglh(modelpara,lambda);

/* Evaluate the Prior distribution
*/
lnprio = priodens(modelpara, pmean, pstdd, pshape);

if __output > 0;
   locate 25,1;
   "Likelihood" lnpy;
endif;

retp(real(-lnpY-lnprio));  /* We minize the inverse of the likelihood fcn */
endp;
